function [soliton,Petv_consts,EL_tests1,EL_tests2] = Petviashvili(beta,N,L,uinit)

% Petviashvili.m - Computes the soliton by the Petviashvili iteration
% (Version without nonradial corrections, to save memory)
%
% By Laurent Demanet, 2005

p = 2*beta + 1;
gamma = p/(p-1);
delta = -1/2;

h = L/N;
if mod(N,2),
    absc = -(L/2-h/2):h:(L/2-h/2);
else
    absc = -L/2:h:(L/2-h);
end
[X,Y,Z] = meshgrid(absc,absc,absc);
R = sqrt(X.^2 + Y.^2 + Z.^2);
clear X Y Z;

xi = 1:N;
xifreq0 = floor(N/2+1);
xi = xi - xifreq0;
[XI1,XI2,XI3] = meshgrid(xi);
absXIsquared = XI1.^2 + XI2.^2 + XI3.^2;
Petv_mult = 1+4*pi^2*absXIsquared/L^2;
clear XI1 XI2 XI3;

if nargin < 4,
    u = 4*exp(-R.^2);
else
    u = uinit;
end
clear R uinit;

prev_u = u;
Petv_consts = [];
EL_tests1 = [];
EL_tests2 = [];
err = 1;
k = 0;

disp('k | Petv_const | Euler-Lagrange remainder');
while (err > 2e-11) & (k < 50)

    k = k+1;
    prevprev_u = prev_u;
    prev_u = u;
    U = fftshift(fftn(ifftshift(u)));
    UP = fftshift(fftn(ifftshift(abs(u).^p)));
    Petv_const = abs(sum(sum(sum(Petv_mult.*(U.*U))))/sum(sum(sum(U.*UP))));
    Petv_consts = [Petv_consts,Petv_const];
    U = UP./Petv_mult * Petv_const^gamma;
    u = abs(fftshift(ifftn(ifftshift(U))));
    clear U UP;
    
    % soliton check
    lapl_U = - 4*pi^2*absXIsquared/L^2 .* fftshift(fftn(ifftshift(u)));
    lapl_u = real(fftshift(ifftn(ifftshift(lapl_U))));
    eulerlagrange1 = lapl_u - u + (u).^p;
    EL_test1 = sqrt(sum(sum(sum(abs(eulerlagrange1).^2)))*h^3) / sqrt(sum(sum(sum(abs(u).^2)))*h^3);
    EL_tests1 = [EL_tests1, EL_test1];
    
    % Componentwise Aitken's acceleration + soliton check
    denom = (u - 2*prev_u + prevprev_u);
    if ~all(all(all(denom))),
        u1 = u;
        k = 50;
    else
        u1 = u - (u - prev_u).^2 ./ denom;
    end;
    lapl_U1 = - 4*pi^2*absXIsquared/L^2 .* fftshift(fftn(ifftshift(u1)));
    lapl_u1 = real(fftshift(ifftn(ifftshift(lapl_U1))));
    eulerlagrange2 = lapl_u1 - u1 + (u1).^p;
    EL_test2 = sqrt(sum(sum(sum(abs(eulerlagrange2).^2)))*h^3) / sqrt(sum(sum(sum(abs(u1).^2)))*h^3);
    EL_tests2 = [EL_tests2, EL_test2];
    clear lapl_U1 lapl_u1 euler_lagrange1;

    disp([num2str(k),' | ', num2str(Petv_const),' | ', num2str(EL_test2)]);
    err = EL_test2;
    
end

acc_check = (err < 1e-11);
soliton = u1;
